import logging
import socket
from datetime import datetime
from sys import getsizeof

import json_log_formatter
from django.utils import timezone


class CustomisedJSONFormatter(json_log_formatter.JSONFormatter):
    def set_extra_correlation_id(self, full_log: dict, extra: dict):
        """Set extra correlation_id"""

        if extra.get("thread_correlation_id", None):
            full_log["correlation_id"] = str(
                extra.get("thread_correlation_id")
            )

        return full_log

    def get_trace_id(self, record: logging.LogRecord):
        """Get trace_id"""
        return record.otelTraceID if hasattr(record, "otelTraceID") else None

    def get_exc_info(
        self, full_log: dict, record: logging.LogRecord
    ):  # pragma: no cover
        """Get exc_info"""
        if record.exc_info or record.levelname.lower() == "exception":
            full_log["exc_info"] = self.formatException(record.exc_info)

        return full_log

    def clean_server_time(self, extra: dict):  # pragma: no cover
        """Clean server time"""
        if extra.get("server_time"):
            del extra["server_time"]

        return extra

    def json_record(
        self, message: str, extra: dict, record: logging.LogRecord
    ) -> dict:
        full_log = {
            "time": datetime.now(
                tz=timezone.get_current_timezone()
            ).isoformat(),
            "trace_id": self.get_trace_id(record=record),
            "correlation_id": record.correlation_id,
            "gn_from": None,
            "gn_version": "1.0",
            "severity": record.levelname,
            "controller": None,
            "request_time": None,
            "request_method": None,
            "request_uri": None,
            "request_length": None,
            "request_query": None,
            "response_status": None,
            "bytes_sent": None,
            "remote_ip": None,
            "user_agent": None,
            "message": message,
            "custom": None,
        }

        if extra.get("request") and not isinstance(
            extra.get("request"), socket.SocketType
        ):
            request = extra.get("request")
            web_log = {
                "correlation_id": record.correlation_id,
                "request_method": request.method,
                "request_uri": request.META.get("PATH_INFO"),
                "request_length": getsizeof(request.data),
                "remote_ip": request.META.get("REMOTE_ADDR"),
                "user_agent": request.META.get("HTTP_USER_AGENT"),
                "request_query": request.META.get("QUERY_STRING"),
                "gn_from": request.META.get("HTTP_GN_FROM"),
            }
            full_log.update(web_log)

            del extra["request"]

        full_log = self.set_extra_correlation_id(
            full_log=full_log, extra=extra
        )

        extra = self.clean_server_time(extra=extra)

        full_log = self.get_exc_info(full_log=full_log, record=record)

        return full_log



import logging
import os
from abc import ABC, abstractmethod
from datetime import datetime

from .event import GnEvent

"""
    Development of the tracker base class in to create a model of
    behavior and ensure modularization between different technologies.

    Tracker is the main class, its a Factory for call the correct tracker type

    - TrackerBase(ABS)
        -TrackerLocalFile(TrackerBase)

    - Tracker
"""

default_logger = logging.getLogger(__name__)


class TrackerBase(ABC):
    """
    A class to represent the abstract classes of a tracker

    ...

    Attributes
    ----------
    tracker_type : str
        type of tracker

    Methods
    -------
    send_event(event: dict):
        Abstract methode for implements how to send
        file for log system tecnology
    """

    def __init__(self, tracker_type: str, app_logger: logging = None) -> None:
        self._tracker_type = tracker_type
        self.logger = app_logger if app_logger else default_logger

    @abstractmethod
    def send_event(self, event: dict):
        """Determine how to tracker send event"""
        pass


class TrackerLocalFile(TrackerBase):
    """
    Classe based in TrackeBase, for write logs in local file system.
    """

    def __init__(
        self,
        path: str = "/tmp/events.log",
        mode: str = "a",
        encoding: str = None,
        app_logger: logging = None,
    ) -> None:
        super().__init__("file", app_logger=app_logger)
        self._path = os.path.abspath(path)
        self._mode = mode
        self._encoding = encoding

    def send_event(self, event: dict) -> bool:
        gn_event = GnEvent(event)

        if gn_event.is_valid():
            try:
                gn_event_to_json = gn_event.to_json()
                self._write_file(gn_event_to_json)

            except Exception as err:
                self.logger.error(
                    "TrackerError: Error to write in file system."
                    f"Error message: {err}"
                )
                return False

            self.logger.info(
                f"Event write in file {self._path} -> {datetime.now()}"
            )
            return True

        else:
            self.logger.warning("Event invalid, verify the struct of event")
            return False

    def _write_file(self, event):
        with open(self._path, mode=self._mode, encoding=self._encoding) as f:
            f.write(event + "\n")


class Tracker:
    """
    Classe that implement factory design for call the correct tracker.

    Methods
    -------
    create_tracker(tracker_type: str, *args, **kwargs):
        Method for call the correct tracker type based
        in tracker_tipe parameter
    """

    @staticmethod
    def create_tracker(
        tracker_type: str, app_logger: logging = None, *args, **kwargs
    ) -> TrackerBase:
        logger = app_logger if app_logger else default_logger

        kwargs["app_logger"] = logger

        if tracker_type == "file":
            return TrackerLocalFile(*args, **kwargs)

        else:
            error_msg = f"Tracker type '{tracker_type}' not supported"
            logger.error(error_msg)
            raise ValueError(error_msg)



import logging
import uuid
from contextvars import ContextVar

correlation_id: ContextVar[uuid.UUID] = ContextVar(
    "correlation_id", default=uuid.uuid4()
)


def gn_context_correlation_id_from_request(func):
    def wraps(*args, **kwargs):
        REQUEST_INDEX = 1
        correlation_id.set(
            args[REQUEST_INDEX].headers.get("Correlation-Id", uuid.uuid4())
        )
        return func(*args, **kwargs)

    return wraps


class ContextFilter(logging.Filter):
    """ "Provides correlation id parameter for the logger"""

    def filter(self, record):
        record.correlation_id = correlation_id.get()
        return True



import json
import logging
import os

import requests
from django.http import HttpResponse

logger = logging.getLogger(__name__)


def validate_external_api_key(function):
    def wrap(request, *args, **kwargs):
        if os.getenv("VALIDATE_EXTERNAL_ACCESS_API_KEY") == "false":
            return function(request, *args, **kwargs)

        api_key = request.META.get("HTTP_AUTHORIZATION")

        if not api_key:
            logger.warning("Request sem header HTTP_AUTHORIZATION")
            raise GNPermissionDenied()

        if api_key == f"Bearer {os.getenv('EXTERNAL_ACCESS_API_KEY')}":
            return function(request, *args, **kwargs)
        else:
            logger.warning("Request com header HTTP_AUTHORIZATION errado")
            raise GNPermissionDenied()

    return wrap


def validate_sns_subscription(function):
    def wrap(request, *args, **kwargs):
        payload = json.loads(request.body.decode("utf-8"))
        message_type = request.META.get("HTTP_X_AMZ_SNS_MESSAGE_TYPE")

        if message_type == "SubscriptionConfirmation":
            subscribe_url = payload.get("SubscribeURL")
            res = requests.get(subscribe_url)
            if res.status_code != 200:
                return HttpResponse(
                    "Invalid verification:\n{res.content}", status=400
                )
            else:
                return HttpResponse("OK")
        return function(request, *args, **kwargs)

    return wrap


class GNPermissionDenied(Exception):
    pass



import os


class ReadDotenv:
    @staticmethod
    def read_dotenv(base_dir=None):
        fname = ".env"
        if base_dir:
            fname = os.path.join(base_dir, ".env")

        try:
            if os.path.exists(fname):
                with open(fname, "r") as file:
                    for line in file:
                        if (
                            not line
                            or len(line) < 2
                            or line.find("=") == -1
                            or line.startswith("#")
                        ):
                            continue
                        property, value = line.strip().split("=", 1)
                        os.environ.setdefault(property, value)
        except Exception:
            pass



import json
from datetime import datetime


class GnEvent:
    """
    Class used to represent the event types, created by GetNinjas systens.

    Methods
    -------
    to_json() -> str:
        return event in json format
    is_valid() -> bool:
        return True if event message have the correct structure.
    """

    def __init__(self, event_dict: dict) -> None:
        self._message = event_dict.get("message")
        self._schema = event_dict.get("schema")
        self._application = event_dict.get("application")
        self._true_timestamp = event_dict.get("true_timestamp", datetime.now())
        self._contexts = event_dict.get("contexts")

    def to_json(self) -> str:
        """Return GnEvent in json format."""

        event_dict = {
            "message": self._transform_dict_to_json_scaped_string(
                self._message
            ),
            "schema": self._schema,
            "application": self._application,
            "true_timestamp": str(
                int(self._true_timestamp.timestamp() * 1000)
            ),
        }

        if self._contexts is not None:
            event_dict[
                "contexts"
            ] = self._transform_dict_to_json_scaped_string(self._contexts)

        return json.dumps(event_dict)

    def _transform_dict_to_json_scaped_string(self, dict_object):
        return json.dumps(dict_object, ensure_ascii=False)

    def is_valid(self) -> bool:
        """Verify GnEvent attributes, return True if is correct format."""

        conditions = []

        conditions.append(isinstance(self._message, dict))
        conditions.append(isinstance(self._schema, str))
        conditions.append(isinstance(self._application, str))
        conditions.append(isinstance(self._true_timestamp, datetime))
        conditions.append(isinstance(self._contexts, (dict, type(None))))

        return_value = True if all(conditions) else False

        return return_value



import json
import logging
import os
import socket
from datetime import datetime
from sys import getsizeof

from django.conf import settings
from django.utils import timezone

BUILTIN_ATTRS = {
    "args",
    "asctime",
    "created",
    "exc_info",
    "exc_text",
    "filename",
    "funcName",
    "levelname",
    "levelno",
    "lineno",
    "module",
    "msecs",
    "message",
    "msg",
    "name",
    "pathname",
    "process",
    "processName",
    "relativeCreated",
    "stack_info",
    "thread",
    "threadName",
}


class JSONFormatter(logging.Formatter):
    """JSON log formatter.

    Usage example::

        import logging

        import json_log_formatter

        json_handler = logging.FileHandler(filename='/var/log/my-log.json')
        json_handler.setFormatter(json_log_formatter.JSONFormatter())

        logger = logging.getLogger('my_json')
        logger.addHandler(json_handler)

        logger.info('Sign up', extra={'referral_code': '52d6ce'})

    The log file will contain the following log record (inline)::

        {
            "message": "Sign up",
            "time": "2015-09-01T06:06:26.524448",
            "referral_code": "52d6ce"
        }

    """

    json_lib = json

    def format(self, record):
        message = record.getMessage()
        extra = self.extra_from_record(record)
        json_record = self.json_record(message, extra, record)
        mutated_record = self.mutate_json_record(json_record)
        # Backwards compatibility: Functions that overwrite this but don't
        # return a new value will return None because they modified the
        # argument passed in.
        if mutated_record is None:
            mutated_record = json_record
        return self.to_json(mutated_record)

    def to_json(self, record):
        """Converts record dict to a JSON string.

        Makes effort to serialize a record (represents an object as a string)
        instead of raising TypeError if json library supports default argument.
        Note, ujson doesn't support it.

        Override this method to change the way dict is converted to JSON.

        """
        try:
            return self.json_lib.dumps(record, default=_json_serializable)
        # ujson doesn't support default argument and raises TypeError.
        except TypeError:
            return self.json_lib.dumps(record)

    def extra_from_record(self, record):
        """Returns `extra` dict you passed to logger.

        The `extra` keyword argument is used to populate the `__dict__` of
        the `LogRecord`.

        """
        return {
            attr_name: record.__dict__[attr_name]
            for attr_name in record.__dict__
            if attr_name not in BUILTIN_ATTRS
        }

    def json_record(
        self, message: str, extra: dict, record: logging.LogRecord
    ) -> dict:
        """Prepares a JSON payload which will be logged.

        Override this method to change JSON log format.

        :param message: Log message, e.g., `logger.info(msg='Sign up')`.
        :param extra: Dictionary that was passed as `extra` param
            `logger.info('Sign up', extra={'referral_code': '52d6ce'})`.
        :param record: `LogRecord` we got from `JSONFormatter.format()`.
        :return: Dictionary which will be passed to JSON lib.

        """
        if not os.getenv("GN_PYTHON_LIB_APP_NAME"):
            raise RuntimeError("GN_PYTHON_LIB_APP_NAME env required.")
        time = datetime.now()

        if settings.USE_TZ:
            time = timezone.localtime().isoformat()

        full_log = {
            "message": message,
            "time": time,
            "client": os.getenv("GN_PYTHON_LIB_APP_NAME"),
            "configuration": "0.1",
            "priority": record.levelname,
            "request_time": "-",
            "request_method": "-",
            "status": "-",
            "scheme": "-",
            "request_uri": "-",
            "request_length": "-",
            "bytes_sent": "-",
            "remote_addr": "-",
            "http_user_agent": "-",
            "requestQuery": "-",  # TODO: lef for compatibility, to be removed
            "request_query": "-",
            "correlation id": "-",
            "headers": "-",
            "custom": "-",
        }

        if extra.get("request"):
            if not isinstance(extra.get("request"), socket.SocketType):
                request = extra.get("request")
                web_log = {
                    "request_time": "-",
                    "status": "-",
                    "bytes_sent": "-",
                    "correlation id": "-",
                    "request_method": request.method,
                    "scheme": request.scheme,
                    "request_uri": request.META.get("PATH_INFO", "-"),
                    "request_length": getsizeof(request.body),
                    "remote_addr": request.META.get("REMOTE_ADDR", "-"),
                    "http_user_agent": request.META.get(
                        "HTTP_USER_AGENT", "-"
                    ),
                    "requestQuery": request.META.get("QUERY_STRING", "-"),
                    "request_query": request.META.get("QUERY_STRING", "-"),
                    "headers": dict(request.headers),
                }
                full_log.update(web_log)

            del extra["request"]

        if extra.get("server_time"):
            del extra["server_time"]

        full_log.update(extra)

        if record.exc_info or record.levelname.lower() == "exception":
            full_log["exc_info"] = self.formatException(record.exc_info)

        return full_log

    def mutate_json_record(self, json_record):
        """Override it to convert fields of `json_record` to needed types.

        Default implementation converts `datetime` to string in ISO8601 format.

        """
        for attr_name in json_record:
            attr = json_record[attr_name]
            if isinstance(attr, datetime):
                json_record[attr_name] = attr.isoformat()
        return json_record


def _json_serializable(obj):
    try:
        return obj.__dict__
    except AttributeError:
        return str(obj)
